"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.typeStat = exports.ResultStatus = void 0;
// Ensure TypeScript is exposed before any files import it
require("./mutations/createExposedTypeScript").requireExposedTypeScript();
const automutate_1 = require("automutate");
const chalk_1 = __importDefault(require("chalk"));
const os_1 = require("os");
const loadPendingOptions_1 = require("./options/loadPendingOptions");
const pluralize_1 = require("./output/pluralize");
const createTypeStatProvider_1 = require("./runtime/createTypeStatProvider");
const collectFileNames_1 = require("./collectFileNames");
var ResultStatus;
(function (ResultStatus) {
    ResultStatus[ResultStatus["Succeeded"] = 0] = "Succeeded";
    ResultStatus[ResultStatus["Failed"] = 1] = "Failed";
    ResultStatus[ResultStatus["ConfigurationError"] = 2] = "ConfigurationError";
})(ResultStatus = exports.ResultStatus || (exports.ResultStatus = {}));
const typeStat = async (argv, output) => {
    const allPendingOptions = await tryLoadingPendingOptions(argv, output);
    if (allPendingOptions instanceof Error || typeof allPendingOptions === "string") {
        return {
            error: allPendingOptions,
            status: ResultStatus.ConfigurationError,
        };
    }
    output.stdout(chalk_1.default.greenBright("🚀 Welcome to TypeStat!"));
    output.stdout([
        chalk_1.default.green(`TypeStat will run through the `),
        chalk_1.default.greenBright(allPendingOptions.length),
        chalk_1.default.green(` options ${pluralize_1.pluralize(allPendingOptions.length, "object")} specified in `),
        chalk_1.default.greenBright(argv.config),
        chalk_1.default.green(` to modify your source code.`),
    ].join(""));
    output.stdout(chalk_1.default.greenBright(`This may take a while...${os_1.EOL}`));
    for (let i = 0; i < allPendingOptions.length; i += 1) {
        // Collect all files to be run on this option iteration from the include glob(s)
        const fileNames = await collectFileNames_1.collectFileNames(argv, process.cwd(), allPendingOptions[i].include);
        if (typeof fileNames !== "object") {
            return {
                error: new Error(`Could not run options object ${i + 1}: ${fileNames ?? `No files included by the 'include' setting were found.`}`),
                status: ResultStatus.Failed,
            };
        }
        output.stdout([
            chalk_1.default.green(`${os_1.EOL}Starting options object `),
            chalk_1.default.greenBright(i + 1),
            chalk_1.default.green(" of "),
            chalk_1.default.greenBright(allPendingOptions.length),
            chalk_1.default.green(`.${os_1.EOL}`),
        ].join(""));
        // Run the mutation providers on those starting options and files
        try {
            await automutate_1.runMutations({
                mutationsProvider: createTypeStatProvider_1.createTypeStatProvider({ ...allPendingOptions[i], fileNames }),
            });
        }
        catch (error) {
            return {
                error: error,
                status: ResultStatus.Failed,
            };
        }
        finally {
            output.stdout([
                chalk_1.default.green(`Finished options object `),
                chalk_1.default.greenBright(i + 1),
                chalk_1.default.green(" of "),
                chalk_1.default.greenBright(allPendingOptions.length),
                chalk_1.default.green(`.${os_1.EOL}`),
            ].join(""));
        }
    }
    return {
        status: ResultStatus.Succeeded,
    };
};
exports.typeStat = typeStat;
const tryLoadingPendingOptions = async (argv, output) => {
    try {
        return await loadPendingOptions_1.loadPendingOptions(argv, output);
    }
    catch (error) {
        return error instanceof Error ? error : new Error(error);
    }
};
//# sourceMappingURL=index.js.map