"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createJavaScriptConfig = void 0;
const imports_1 = require("./imports");
const renames_1 = require("./renames");
const createJavaScriptConfig = ({ imports, project, sourceFiles, renames }) => {
    const fileConversion = {
        files: {
            renameExtensions: printRenames(renames),
        },
    };
    const fixConversion = {
        fixes: {
            incompleteTypes: true,
            missingProperties: true,
            noImplicitAny: true,
        },
    };
    const shared = (include) => ({
        ...(include && { include }),
        project: project.filePath,
    });
    const allConversions = imports === imports_1.InitializationImports.Yes
        ? [
            {
                ...fileConversion,
                fixes: {
                    importExtensions: true,
                },
                ...shared(sourceFiles ? [sourceFiles] : undefined),
            },
            {
                ...fixConversion,
                ...shared(sourceFiles
                    ? renames === renames_1.InitializationRenames.Auto
                        ? renameSourceFilesExtensionsAuto(sourceFiles)
                        : renames === renames_1.InitializationRenames.TS
                            ? renameSourceFilesExtensions(sourceFiles, "ts")
                            : renameSourceFilesExtensions(sourceFiles, "tsx")
                    : undefined),
            },
        ]
        : {
            ...fileConversion,
            ...fixConversion,
            ...shared(sourceFiles ? [sourceFiles] : undefined),
        };
    return allConversions;
};
exports.createJavaScriptConfig = createJavaScriptConfig;
const printRenames = (renames) => {
    switch (renames) {
        case renames_1.InitializationRenames.Auto:
            return true;
        case renames_1.InitializationRenames.TS:
            return "ts";
        case renames_1.InitializationRenames.TSX:
            return "tsx";
    }
};
function renameSourceFilesExtensionsAuto(sourceFiles) {
    for (const [original, replacement] of [
        [/\{js,jsx\}/, "{ts,tsx}"],
        [/\.js$/, ".{ts,tsx}"],
    ]) {
        sourceFiles = sourceFiles.replace(original, replacement);
    }
    return renameSourceFilesExtensions(sourceFiles, "ts");
}
function renameSourceFilesExtensions(sourceFiles, extension) {
    return [sourceFiles.replace(/(\.|{)js/, `$1${extension}`).replace(new RegExp(`{${extension},jsx?}`), extension)];
}
//# sourceMappingURL=createJavaScriptConfig.js.map