"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const imports_1 = require("./imports");
const renames_1 = require("./renames");
const createJavaScriptConfig_1 = require("./createJavaScriptConfig");
describe(createJavaScriptConfig_1.createJavaScriptConfig, () => {
    test.each([
        {
            expected: {
                files: { renameExtensions: "ts" },
                fixes: { incompleteTypes: true, missingProperties: true, noImplicitAny: true },
                project: "tsconfig.json",
            },
            name: "Basic",
            settings: {
                imports: imports_1.InitializationImports.No,
                project: {
                    filePath: "tsconfig.json",
                },
                renames: renames_1.InitializationRenames.TS,
            },
        },
        {
            expected: [
                {
                    files: { renameExtensions: "ts" },
                    fixes: { importExtensions: true },
                    include: ["src/**/*.{js,jsx}"],
                    project: "tsconfig.json",
                },
                {
                    fixes: { incompleteTypes: true, missingProperties: true, noImplicitAny: true },
                    include: ["src/**/*.ts"],
                    project: "tsconfig.json",
                },
            ],
            name: "TS Renames (multiple sourceFiles extensions)",
            settings: {
                imports: imports_1.InitializationImports.Yes,
                project: {
                    filePath: "tsconfig.json",
                },
                renames: renames_1.InitializationRenames.TS,
                sourceFiles: "src/**/*.{js,jsx}",
            },
        },
        {
            expected: [
                {
                    files: { renameExtensions: "tsx" },
                    fixes: { importExtensions: true },
                    include: ["src/**/*.{js,jsx}"],
                    project: "tsconfig.json",
                },
                {
                    fixes: { incompleteTypes: true, missingProperties: true, noImplicitAny: true },
                    include: ["src/**/*.tsx"],
                    project: "tsconfig.json",
                },
            ],
            name: "TSX Renames (multiple sourceFiles extensions)",
            settings: {
                imports: imports_1.InitializationImports.Yes,
                project: {
                    filePath: "tsconfig.json",
                },
                renames: renames_1.InitializationRenames.TSX,
                sourceFiles: "src/**/*.{js,jsx}",
            },
        },
        {
            expected: [
                { files: { renameExtensions: true }, fixes: { importExtensions: true }, project: "tsconfig.json" },
                { fixes: { incompleteTypes: true, missingProperties: true, noImplicitAny: true }, project: "tsconfig.json" },
            ],
            name: "Auto Renames (no sourceFiles)",
            settings: {
                imports: imports_1.InitializationImports.Yes,
                project: {
                    filePath: "tsconfig.json",
                },
                renames: renames_1.InitializationRenames.Auto,
            },
        },
        {
            expected: [
                {
                    files: { renameExtensions: true },
                    fixes: { importExtensions: true },
                    include: ["src/**/*.js"],
                    project: "tsconfig.json",
                },
                {
                    fixes: { incompleteTypes: true, missingProperties: true, noImplicitAny: true },
                    include: ["src/**/*.{ts,tsx}"],
                    project: "tsconfig.json",
                },
            ],
            name: "Auto Renames (single sourceFiles extension)",
            settings: {
                imports: imports_1.InitializationImports.Yes,
                project: {
                    filePath: "tsconfig.json",
                },
                renames: renames_1.InitializationRenames.Auto,
                sourceFiles: "src/**/*.js",
            },
        },
        {
            expected: [
                {
                    files: { renameExtensions: true },
                    fixes: { importExtensions: true },
                    include: ["src/**/*.{js,jsx}"],
                    project: "tsconfig.json",
                },
                {
                    fixes: { incompleteTypes: true, missingProperties: true, noImplicitAny: true },
                    include: ["src/**/*.{ts,tsx}"],
                    project: "tsconfig.json",
                },
            ],
            name: "Auto Renames (multiple sourceFiles extensions)",
            settings: {
                imports: imports_1.InitializationImports.Yes,
                project: {
                    filePath: "tsconfig.json",
                },
                renames: renames_1.InitializationRenames.Auto,
                sourceFiles: "src/**/*.{js,jsx}",
            },
        },
        {
            expected: [
                {
                    files: { renameExtensions: true },
                    fixes: { importExtensions: true },
                    include: ["src/**/*.js(x)"],
                    project: "tsconfig.json",
                },
                {
                    fixes: { incompleteTypes: true, missingProperties: true, noImplicitAny: true },
                    include: ["src/**/*.ts(x)"],
                    project: "tsconfig.json",
                },
            ],
            name: "Auto Renames (parenthesized sourceFiles extensions)",
            settings: {
                imports: imports_1.InitializationImports.Yes,
                project: {
                    filePath: "tsconfig.json",
                },
                renames: renames_1.InitializationRenames.Auto,
                sourceFiles: "src/**/*.js(x)",
            },
        },
    ])("$name", async ({ expected, settings }) => {
        const actual = await createJavaScriptConfig_1.createJavaScriptConfig(settings);
        expect(actual).toEqual(expected);
    });
});
//# sourceMappingURL=createJavaScriptConfig.test.js.map