"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.initializeNewProject = void 0;
const mz_1 = require("mz");
const enquirer_1 = require("enquirer");
const filePath = "./tsconfig.json";
const initializeNewProject = async () => {
    const { emit } = await enquirer_1.prompt([
        {
            choices: ["yes", "no"],
            message: "Should TypeScript output .js files from .ts sources?",
            name: "emit",
            type: "select",
        },
    ]);
    const { inBrowser } = await enquirer_1.prompt([
        {
            choices: ["yes", "no"],
            message: "Does your code run in browser?",
            name: "inBrowser",
            type: "select",
        },
    ]);
    const jsx = inBrowser &&
        (await enquirer_1.prompt([
            {
                choices: ["yes", "no"],
                message: "Does your project use JSX?",
                name: "jsx",
                type: "select",
            },
        ])).jsx;
    const { target } = await enquirer_1.prompt([
        {
            choices: ["es2020", "es2021", "es2022", "esnext"],
            message: "What minimum runtime does your code run on?",
            name: "target",
            type: "select",
        },
    ]);
    const { strict } = await enquirer_1.prompt([
        {
            choices: ["yes", "no"],
            message: "Would you like to enable TypeScript's strict compiler options? (recommended)",
            name: "strict",
            type: "select",
        },
    ]);
    await mz_1.fs.writeFile(filePath, JSON.stringify({
        compilerOptions: {
            declaration: true,
            declarationMap: true,
            ...(emit === "no" && { noEmit: true }),
            esModuleInterop: true,
            ...(inBrowser === "no" && { lib: [target] }),
            ...(jsx === "yes" && { jsx: "react" }),
            sourceMap: true,
            moduleResolution: "node",
            module: "nodenext",
            skipLibCheck: true,
            ...(strict === "yes" ? { strict: true } : {}),
            target,
        },
    }, null, 4));
    return { created: true, filePath };
};
exports.initializeNewProject = initializeNewProject;
//# sourceMappingURL=initializeNewProject.js.map