"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeMultiTypeScriptConfig = void 0;
const mz_1 = require("mz");
const improvements_1 = require("./improvements");
const writeMultiTypeScriptConfig = async ({ fileName, improvements, project, sourceFiles, testFiles, }) => {
    await mz_1.fs.writeFile(fileName, JSON.stringify([
        {
            fixes: {
                ...printImprovements(improvements),
                strictNonNullAssertions: true,
            },
            include: [testFiles],
            projectPath: project.filePath,
            types: {
                strictNullChecks: true,
            },
        },
        {
            exclude: [testFiles],
            fixes: printImprovements(improvements),
            ...(sourceFiles && { include: [sourceFiles] }),
            projectPath: project.filePath,
        },
        {
            fixes: printImprovements(improvements),
            include: [testFiles, sourceFiles],
            projectPath: project.filePath,
        },
    ], undefined, 4));
};
exports.writeMultiTypeScriptConfig = writeMultiTypeScriptConfig;
const printImprovements = (improvements) => {
    return {
        incompleteTypes: true,
        ...(improvements.has(improvements_1.InitializationImprovement.NoImplicitAny) && { noImplicitAny: true }),
        ...(improvements.has(improvements_1.InitializationImprovement.NoInferableTypes) && { inferableTypes: true }),
        ...(improvements.has(improvements_1.InitializationImprovement.NoImplicitThis) && { noImplicitThis: true }),
    };
};
//# sourceMappingURL=writeMultiTypeScriptConfig.js.map