"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.initializeSources = void 0;
const enquirer_1 = require("enquirer");
const mz_1 = require("mz");
const everything = "everything";
const other = "other";
const initializeSources = async (settings) => {
    const completion = settings.fromJavaScript ? "/**/*.{js,jsx}" : "/**/*.{ts,tsx}";
    const builtIn = await initializeBuiltInSources(completion, settings.project);
    if (builtIn === other) {
        return await getCustomSources(completion);
    }
    if (builtIn === everything) {
        return undefined;
    }
    if (settings.project.created) {
        await mz_1.fs.writeFile(settings.project.filePath, JSON.stringify({
            ...JSON.parse((await mz_1.fs.readFile(settings.project.filePath)).toString()),
            include: [builtIn.replace(/\*\.\{(j|t)s\,(j|t)sx\}/, "").replace(/\/\*\*\/$/, "")],
        }, null, 4));
    }
    return builtIn;
};
exports.initializeSources = initializeSources;
const initializeBuiltInSources = async (completion, project) => {
    // https://github.com/enquirer/enquirer/issues/202
    const choices = [
        {
            message: "everything in my tsconfig.json",
            name: everything,
        },
        `lib${completion}`,
        `src${completion}`,
        other,
    ];
    const { sourceFiles } = await enquirer_1.prompt([
        {
            choices,
            initial: choices.length - 1,
            message: "Which glob matches files you'd like to convert?",
            name: "sourceFiles",
            type: "select",
        },
    ]);
    return sourceFiles;
};
const getCustomSources = async (completion) => {
    const { sourceFiles } = await enquirer_1.prompt([
        {
            initial: `src${completion}`,
            message: "Which files would you like to convert?",
            name: "sourceFiles",
            type: "text",
        },
    ]);
    return sourceFiles;
};
//# sourceMappingURL=index.js.map