"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTypeCreationMutation = exports.createTypeAdditionMutation = void 0;
const tsutils = __importStar(require("tsutils"));
const ts = __importStar(require("typescript"));
const nodeTypes_1 = require("../shared/nodeTypes");
const joinIntoType_1 = require("./aliasing/joinIntoType");
const collecting_1 = require("./collecting");
/**
 * Creates a mutation to add types to an existing type, if any are new.
 *
 * @param request   Source file, metadata, and settings to collect mutations in the file.
 * @param node   Original node with a type declaration to add to.
 * @param declaredType   Declared type from the node.
 * @param allAssignedTypes   Types now assigned to the node.
 * @returns Mutation to add any new assigned types, if any are missing from the declared type.
 */
const createTypeAdditionMutation = (request, node, declaredType, allAssignedTypes) => {
    // Find any missing flags and symbols (a.k.a. types)
    const { missingFlags, missingTypes } = collecting_1.collectUsageFlagsAndSymbols(request, declaredType, allAssignedTypes);
    // If nothing is missing, rejoice! The type was already fine.
    if (missingFlags.size === 0 && missingTypes.size === 0) {
        return undefined;
    }
    // Join the missing types into a type string to declare
    const newTypeAlias = joinIntoType_1.joinIntoType(missingFlags, missingTypes, request);
    if (newTypeAlias === undefined) {
        return undefined;
    }
    // If the original type was a bottom type or just something like Function or Object, replace it entirely
    if (tsutils.isTypeFlagSet(declaredType, ts.TypeFlags.Never | ts.TypeFlags.Unknown) || nodeTypes_1.isKnownGlobalBaseType(declaredType)) {
        return {
            insertion: ` ${newTypeAlias}`,
            range: {
                begin: node.type.pos,
                end: node.type.end,
            },
            type: "text-swap",
        };
    }
    // Create a mutation insertion that adds the missing types in
    return {
        insertion: ` | ${newTypeAlias}`,
        range: {
            begin: node.type.end,
        },
        type: "text-insert",
    };
};
exports.createTypeAdditionMutation = createTypeAdditionMutation;
/**
 * Creates a mutation to add types to a node without a type, if any are new.
 *
 * @param request   Metadata and settings to collect mutations in a file.
 * @param node   Node to add the type annotation.
 * @param declaredType   Declared type from the node.
 * @param allAssignedTypes   Types now assigned to the node.
 * @returns Mutation to add any new assigned types, if any are missing from the declared type.
 */
const createTypeCreationMutation = (request, node, declaredType, allAssignedTypes) => {
    // Find the already assigned flags and symbols, as well as any missing ones
    const { assignedFlags, assignedTypes, missingFlags, missingTypes } = collecting_1.collectUsageFlagsAndSymbols(request, declaredType, allAssignedTypes);
    // If nothing is missing, rejoice! The type was already fine.
    if (missingFlags.size === 0 && missingTypes.size === 0) {
        return undefined;
    }
    // Join the missing types into a type string to declare
    const newTypeAlias = joinIntoType_1.joinIntoType(assignedFlags, assignedTypes, request);
    if (newTypeAlias === undefined) {
        return undefined;
    }
    // Create a mutation insertion that adds the assigned types in
    return {
        insertion: `: ${newTypeAlias}`,
        range: {
            begin: node.name.end,
        },
        type: "text-insert",
    };
};
exports.createTypeCreationMutation = createTypeCreationMutation;
//# sourceMappingURL=creators.js.map