"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.originalTypeHasIncompleteType = void 0;
const tsutils = __importStar(require("tsutils"));
const ts = __importStar(require("typescript"));
const nodeTypes_1 = require("../../shared/nodeTypes");
const onlyTypes = (candidateTypes) => !candidateTypes.some((candidateType) => typeof candidateType === "string");
/**
 * @returns Whether any of the extra types don't yet exist on an original type.
 * @remarks If any of the candidate types are strings, this unfortunately has to assume true.
 */
const originalTypeHasIncompleteType = (request, originalType, candidateTypes) => {
    if (!onlyTypes(candidateTypes)) {
        return true;
    }
    // If the original type is something like Function and at least one candidate type isn't,
    // consider the Function to be reporting not enough info (like a base type)
    if (nodeTypes_1.isKnownGlobalBaseType(originalType) && !candidateTypes.every(nodeTypes_1.isKnownGlobalBaseType)) {
        return true;
    }
    // If the original type is unknown or never, we can always assume it's missing info
    if (nodeTypes_1.isNeverAndOrUnknownType(originalType)) {
        return true;
    }
    return candidateTypes.some((assignedType) => !candidateTypeIsAssignableToOriginal(request, assignedType, originalType));
};
exports.originalTypeHasIncompleteType = originalTypeHasIncompleteType;
const candidateTypeIsAssignableToOriginal = (request, candidateType, originalType) => {
    const typeChecker = request.services.program.getTypeChecker();
    // The type checker things that functions with similar base return types are the same
    // e.g. () => boolean is marked as assignable to () => void
    // We know that's false, so if the two are functions
    const missingFunctionReturn = functionReturnIsIncomplete(request, candidateType, originalType);
    if (missingFunctionReturn !== undefined) {
        return !missingFunctionReturn;
    }
    // Otherwise we can directly use isTypeAssignableTo checking
    return typeChecker.isTypeAssignableTo(candidateType, originalType);
};
const functionReturnIsIncomplete = (request, candidateType, originalType) => {
    const typeChecker = request.services.program.getTypeChecker();
    // Skip this logic if neither of the types are actually functions that return void
    if (!anySignatureReturnsVoid(candidateType) && !anySignatureReturnsVoid(originalType)) {
        return undefined;
    }
    // Regardless of the original compiler options, factor in covariance checks to be super duper sure
    return !typeChecker.isTypeAssignableTo(candidateType, originalType) || !typeChecker.isTypeAssignableTo(originalType, candidateType);
};
function anySignatureReturnsVoid(type) {
    return type.getCallSignatures().some((callSignature) => tsutils.isTypeFlagSet(callSignature.getReturnType(), ts.TypeFlags.Void));
}
//# sourceMappingURL=eliminations.js.map