"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fixReactPropsMissing = void 0;
const automutate_1 = require("automutate");
const ts = __importStar(require("typescript"));
const collectMutationsFromNodes_1 = require("../../../collectMutationsFromNodes");
const isReactComponentNode_1 = require("./reactFiltering/isReactComponentNode");
const getComponentPropsNode_1 = require("./getComponentPropsNode");
const getApparentNameOfComponent_1 = require("./getApparentNameOfComponent");
const newlines_1 = require("../../../../shared/printing/newlines");
/**
 * Creates a new props type for a React component from scratch.
 */
const fixReactPropsMissing = (request) => {
    return collectMutationsFromNodes_1.collectMutationsFromNodes(request, isReactComponentNode_1.isReactComponentNode, visitReactComponentNode);
};
exports.fixReactPropsMissing = fixReactPropsMissing;
const visitReactComponentNode = (node, request) => {
    // Make sure a node doesn't yet exist to declare the node's props type
    const propsNode = getComponentPropsNode_1.getComponentPropsNode(request, node);
    if (propsNode !== undefined) {
        return undefined;
    }
    // Find all types of props later passed to the node
    const attributeTypesAndRequirements = collectComponentAttributeTypes(request, node);
    if (!attributeTypesAndRequirements?.attributeTypes.size) {
        return undefined;
    }
    // Generate a name for a new props interface to use on the node
    const interfaceName = `${getApparentNameOfComponent_1.getApparentNameOfComponent(request, node)}Props`;
    return automutate_1.combineMutations(
    // That interface will be injected with blank lines around it just before the component
    createPropsTypeCreationMutation(request, node, interfaceName, attributeTypesAndRequirements), 
    // We'll also annotate the component with a type declaration to use the new prop type
    createPropsTypeUsageMutation(node, interfaceName));
};
const collectComponentAttributeTypes = (request, node) => {
    // Find all references to the node, if there are more than just the node itself
    const references = request.fileInfoCache.getNodeReferencesAsNodes(node.parent.kind === ts.SyntaxKind.VariableDeclaration ? node.parent : node);
    if (references === undefined || references.length === 0) {
        return undefined;
    }
    const allAttributeNames = new Set();
    const allAttributeNameUses = [];
    const attributeTypes = new Map();
    // For each reference, try to collect attribute type from its usage...
    for (const reference of references) {
        // ...assuming the reference is a JSX element used with attributes
        const attributesElement = getAttributesElement(reference);
        if (attributesElement === undefined) {
            continue;
        }
        // Keep track of attribute names that are used, so later we can figure out which are optional
        const usedAttributeNames = [];
        for (const attribute of attributesElement.attributes.properties) {
            // Only look at JSX attributes, like prop={value} and prop
            if (!ts.isJsxAttribute(attribute)) {
                continue;
            }
            // Attempt to retrieve the type of the prop's value
            const typeChecker = request.services.program.getTypeChecker();
            const valueType = getAttributeValueType(typeChecker, attribute);
            if (valueType === undefined) {
                continue;
            }
            // Widen literals such as `false` to their primitives such as `boolean`
            const type = typeChecker.getBaseTypeOfLiteralType(valueType);
            // Add the type underneath the attribute name
            const name = attribute.name.text;
            const types = attributeTypes.get(name);
            if (types === undefined) {
                attributeTypes.set(name, [type]);
            }
            else {
                types.push(type);
            }
            // Remember the attribute name in all attributes, and in this usage (element)
            allAttributeNames.add(name);
            usedAttributeNames.push(name);
        }
        allAttributeNameUses.push(new Set(usedAttributeNames));
    }
    // Mark only the attributes that appear in every usage as required
    const requiredAttributeNames = new Set(Array.from(allAttributeNames).filter((attributeName) => allAttributeNameUses.every((attributeNameUses) => attributeNameUses.has(attributeName))));
    return { attributeTypes, requiredAttributeNames };
};
const getAttributesElement = (reference) => {
    if (!ts.isIdentifier(reference)) {
        return undefined;
    }
    const { parent } = reference;
    if (ts.isJsxElement(parent)) {
        return parent.openingElement;
    }
    if (ts.isJsxSelfClosingElement(parent)) {
        return parent;
    }
    return undefined;
};
const getAttributeValueType = (typeChecker, attribute) => {
    if (attribute.initializer) {
        if (ts.isStringLiteral(attribute.initializer)) {
            return typeChecker.getTypeAtLocation(attribute);
        }
        return ts.isJsxExpression(attribute.initializer) && attribute.initializer.expression
            ? typeChecker.getTypeAtLocation(attribute.initializer.expression)
            : undefined;
    }
    return typeChecker.getTypeAtLocation(attribute.name);
};
const createPropsTypeCreationMutation = (request, node, interfaceName, { attributeTypes, requiredAttributeNames }) => {
    const endline = newlines_1.printNewLine(request.options.compilerOptions);
    return {
        insertion: [
            endline,
            `interface ${interfaceName} {`,
            ...Array.from(attributeTypes).map(([name, type]) => `${name}${requiredAttributeNames.has(name) ? "" : "?"}: ${request.services.printers.type(type)};`),
            `}`,
        ].join(endline),
        range: {
            begin: ts.isClassDeclaration(node)
                ? node.pos
                : ts.isVariableDeclaration(node.parent)
                    ? node.parent.parent.pos
                    : node.parent.pos,
        },
        type: "text-insert",
    };
};
const createPropsTypeUsageMutation = (node, interfaceName) => {
    return ts.isFunctionLike(node)
        ? {
            insertion: `: ${interfaceName}`,
            range: {
                begin: node.parameters[0].end,
            },
            type: "text-insert",
        }
        : {
            insertion: `<${interfaceName}>`,
            range: {
                begin: node.heritageClauses[0].end,
            },
            type: "text-insert",
        };
};
//# sourceMappingURL=fixReactPropsMissing.js.map