"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fillOutRawOptions = void 0;
const arrays_1 = require("../shared/arrays");
const enums_1 = require("./enums");
const collectAddedMutators_1 = require("./parsing/collectAddedMutators");
const collectFileOptions_1 = require("./parsing/collectFileOptions");
const collectNoImplicitAny_1 = require("./parsing/collectNoImplicitAny");
const collectNoImplicitThis_1 = require("./parsing/collectNoImplicitThis");
const collectPackageOptions_1 = require("./parsing/collectPackageOptions");
const collectStrictNullChecks_1 = require("./parsing/collectStrictNullChecks");
/**
 * Combines Node and CLi argument options with project and file metadata into pending TypeStat options.
 *
 * @returns Parsed TypeStat options, or a string for an error complaint.
 */
const fillOutRawOptions = ({ compilerOptions, cwd, output, projectPath, rawOptions, }) => {
    const rawOptionTypes = rawOptions.types === undefined ? {} : rawOptions.types;
    const noImplicitAny = collectNoImplicitAny_1.collectNoImplicitAny(compilerOptions, rawOptions);
    const noImplicitThis = collectNoImplicitThis_1.collectNoImplicitThis(compilerOptions, rawOptions);
    const { compilerStrictNullChecks, typeStrictNullChecks } = collectStrictNullChecks_1.collectStrictNullChecks(compilerOptions, rawOptionTypes);
    const packageOptions = collectPackageOptions_1.collectPackageOptions(cwd, rawOptions);
    const shell = [];
    if (rawOptions.postProcess !== undefined && rawOptions.postProcess.shell !== undefined) {
        shell.push(...rawOptions.postProcess.shell);
    }
    return {
        compilerOptions: {
            ...compilerOptions,
            noImplicitAny,
            noImplicitThis,
            strictNullChecks: compilerStrictNullChecks,
        },
        files: collectFileOptions_1.collectFileOptions(rawOptions),
        filters: arrays_1.collectOptionals(rawOptions.filters),
        fixes: {
            importExtensions: false,
            incompleteTypes: false,
            missingProperties: false,
            noImplicitAny: false,
            noImplicitThis: false,
            noInferableTypes: false,
            strictNonNullAssertions: false,
            ...rawOptions.fixes,
        },
        hints: {
            react: {
                propTypes: rawOptions.hints?.react?.propTypes ?? enums_1.ReactPropTypesHint.WhenRequired,
                propTypesOptionality: rawOptions.hints?.react?.propTypesOptionality ?? enums_1.ReactPropTypesOptionality.AsWritten,
            },
        },
        include: rawOptions.include ?? compilerOptions.include,
        mutators: collectAddedMutators_1.collectAddedMutators(rawOptions, packageOptions.directory, output),
        output,
        package: packageOptions,
        postProcess: { shell },
        projectPath,
        types: {
            strictNullChecks: typeStrictNullChecks,
        },
    };
};
exports.fillOutRawOptions = fillOutRawOptions;
//# sourceMappingURL=fillOutRawOptions.js.map