"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadPendingOptions = void 0;
const path = __importStar(require("path"));
const paths_1 = require("../shared/paths");
const fillOutRawOptions_1 = require("./fillOutRawOptions");
const findRawOptions_1 = require("./findRawOptions");
const optionVerification_1 = require("./optionVerification");
const parseRawCompilerOptions_1 = require("./parseRawCompilerOptions");
/**
 * Reads pre-file-rename TypeStat options using a config path.
 *
 * @param argv   Root arguments passed to TypeStat.
 * @param output   Wraps process and logfile output.
 * @returns Promise for filled-out TypeStat options, or a string complaint from failing to make them.
 */
const loadPendingOptions = async (argv, output) => {
    if (argv.config === undefined) {
        return "-c/--config file must be provided.";
    }
    const cwd = process.cwd();
    const foundRawOptions = findRawOptions_1.findRawOptions(cwd, argv.config);
    if (typeof foundRawOptions === "string") {
        return foundRawOptions;
    }
    const { allRawOptions, filePath } = foundRawOptions;
    const results = [];
    // Fill out each option in the config with its own separate settings
    // (or return the first failure string describing which one is incorrect)
    for (let i = 0; i < allRawOptions.length; i += 1) {
        const rawOptions = allRawOptions[i];
        const projectPath = getProjectPath(cwd, filePath, rawOptions);
        const compilerOptions = await parseRawCompilerOptions_1.parseRawCompilerOptions(cwd, projectPath);
        const filledOutOptions = fillOutRawOptions_1.fillOutRawOptions({ argv, compilerOptions, cwd, output, projectPath, rawOptions });
        const complaint = optionVerification_1.findComplaintForOptions(filledOutOptions);
        if (complaint) {
            return `Invalid options at index ${i}: ${complaint}`;
        }
        results.push(filledOutOptions);
    }
    return results;
};
exports.loadPendingOptions = loadPendingOptions;
const getProjectPath = (cwd, filePath, rawOptions) => {
    // If the TypeStat configuration file has a projectPath field, use that relative to the file
    if (filePath !== undefined && rawOptions.projectPath !== undefined) {
        return paths_1.normalizeAndSlashify(path.join(path.dirname(filePath), rawOptions.projectPath));
    }
    // Otherwise give up and try a ./tsconfig.json relative to the package directory
    return paths_1.normalizeAndSlashify(path.join(cwd, "tsconfig.json"));
};
//# sourceMappingURL=loadPendingOptions.js.map