"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createProviderFromProviders = void 0;
/**
 * Creates a provider that runs through a series of providers.
 *
 * @param providers   Providers to return changes from, in order.
 * @returns Single provider equivalent to the given providers, in order.
 */
const createProviderFromProviders = (options, providerCreators) => {
    let index = -1;
    let provider;
    const multiProvider = async () => {
        if (!provider) {
            for (index += 1; index < providerCreators.length; index += 1) {
                provider = providerCreators[index](options);
                if (provider) {
                    break;
                }
            }
            if (!provider) {
                return { fileMutations: undefined };
            }
        }
        const results = await provider();
        if (!results) {
            provider = undefined;
            return multiProvider();
        }
        options = results.newOptions ?? options;
        if (results.mutationsWave?.fileMutations === undefined) {
            return { fileMutations: undefined };
        }
        return results.mutationsWave;
    };
    return multiProvider;
};
exports.createProviderFromProviders = createProviderFromProviders;
//# sourceMappingURL=createProviderFromProviders.js.map