"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createSingleUseProvider = void 0;
const chalk_1 = __importDefault(require("chalk"));
const os_1 = require("os");
/**
 * Creates a provider that will run exactly once.
 *
 * @param message   Message to log before and after running.
 * @param provider   Provider to wrap around.
 * @returns Single-use equivalent of the provider.
 */
const createSingleUseProvider = (message, providerCreator) => {
    return (options) => {
        const provider = providerCreator(options);
        if (!provider) {
            return undefined;
        }
        let provided = false;
        return async () => {
            if (provided) {
                return undefined;
            }
            options.output.stdout(chalk_1.default.blue(`${message}...`));
            const result = await provider();
            options.output.stdout(chalk_1.default.blueBright(`Done.${os_1.EOL}`));
            provided = true;
            return result;
        };
    };
};
exports.createSingleUseProvider = createSingleUseProvider;
//# sourceMappingURL=createSingleUseProvider.js.map