"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createMarkFilesModifiedProvider = void 0;
const fs = __importStar(require("mz/fs"));
const newlines_1 = require("../../shared/printing/newlines");
const createSingleUseProvider_1 = require("../createSingleUseProvider");
/**
 * Creates a mutations provider to mark all previously mutated files as modified.
 *
 * @returns Mutations provider to mark all mutated files as modified, if needed.
 */
const createMarkFilesModifiedProvider = (allModifiedFileNames) => {
    return createSingleUseProvider_1.createSingleUseProvider("Marking files as modified", (options) => {
        if (options.files.above === "" && options.files.below === "") {
            return undefined;
        }
        return async () => {
            const fileMutations = {};
            let hadMutation = false;
            for (const fileName of allModifiedFileNames) {
                const mutations = await createFileMutations(options, fileName);
                if (mutations.length !== 0) {
                    fileMutations[fileName] = mutations;
                    hadMutation = true;
                }
            }
            return {
                mutationsWave: {
                    fileMutations: hadMutation ? fileMutations : undefined,
                },
            };
        };
    });
};
exports.createMarkFilesModifiedProvider = createMarkFilesModifiedProvider;
const createFileMutations = async (options, fileName) => {
    const mutations = [];
    const fileContents = (await fs.readFile(fileName)).toString();
    const fileContentsTrimmed = fileContents.trim();
    const newLine = newlines_1.printNewLine(options.compilerOptions);
    if (options.files.above !== "" && !fileContentsTrimmed.startsWith(options.files.above)) {
        mutations.push({
            insertion: `${options.files.above}${newLine}`,
            range: {
                begin: 0,
                end: 0,
            },
            type: "text-insert",
        });
    }
    if (options.files.below !== "" && !fileContentsTrimmed.endsWith(options.files.below)) {
        mutations.push({
            insertion: `${newLine}${options.files.below}`,
            range: {
                begin: getInsertionIndexBeforeLastEndline(fileContents),
            },
            type: "text-insert",
        });
    }
    return mutations;
};
const getInsertionIndexBeforeLastEndline = (fileContents) => {
    if (fileContents.length === 0) {
        return 0;
    }
    let index = fileContents.length - 1;
    for (const character of ["\n", "\r"]) {
        if (fileContents[index] === character) {
            index -= 1;
            if (index === 0) {
                return 0;
            }
        }
    }
    return index + 1;
};
//# sourceMappingURL=createMarkFilesModifiedProvider.js.map