"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createPostProcessingProvider = void 0;
const createSingleUseProvider_1 = require("../createSingleUseProvider");
const runCommand_1 = require("./missingTypes/runCommand");
/**
 * Creates a mutations provider to run any post-processing shell scripts.
 *
 * @param allModifiedFilePaths   Unique names of all files that were modified.
 * @returns Mutations provider to run post-processing shell scripts, if needed.
 */
const createPostProcessingProvider = (allModifiedFilePaths) => {
    return createSingleUseProvider_1.createSingleUseProvider("Running post-processing scripts", (options) => {
        if (options.postProcess.shell.length === 0) {
            return undefined;
        }
        return async () => {
            for (const shellCommand of options.postProcess.shell) {
                await runCommand_1.runCommand(options, [...shellCommand, ...Array.from(allModifiedFilePaths)]);
            }
            return {
                mutationsWave: {},
            };
        };
    });
};
exports.createPostProcessingProvider = createPostProcessingProvider;
//# sourceMappingURL=createPostProcessingProvider.js.map