"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createRequireRenameProvider = void 0;
const findRequireRenameMutationsInFile_1 = require("../../mutations/renames/findRequireRenameMutationsInFile");
const maps_1 = require("../../shared/maps");
const createFileNamesAndServices_1 = require("../createFileNamesAndServices");
const createSingleUseProvider_1 = require("../createSingleUseProvider");
/**
 * Creates a mutations provider that transforms local require() calls in files.
 *
 * @param allModifiedFileNames   Set to mark names of all files that were modified.
 * @returns Provider to transform local require() calls in files, if needed.
 */
const createRequireRenameProvider = (allModifiedFiles) => {
    return createSingleUseProvider_1.createSingleUseProvider("Transforming local require() calls in files", (options) => {
        if (!options.files.renameExtensions) {
            return undefined;
        }
        return async () => {
            const fileMutations = new Map();
            const { fileNames, services } = createFileNamesAndServices_1.createFileNamesAndServices(options);
            const allFileNames = new Set(fileNames);
            for (const fileName of fileNames) {
                const sourceFile = services.program.getSourceFile(fileName);
                if (sourceFile === undefined) {
                    options.output.stderr(`Could not find TypeScript source file for '${fileName}'.`);
                    continue;
                }
                const foundMutations = findRequireRenameMutationsInFile_1.findRequireRenameMutationsInFile({
                    allFileNames,
                    options,
                    sourceFile,
                });
                if (foundMutations.length !== 0) {
                    allModifiedFiles.add(fileName);
                    fileMutations.set(fileName, foundMutations);
                }
            }
            return {
                mutationsWave: {
                    fileMutations: fileMutations.size === 0 ? undefined : maps_1.convertMapToObject(fileMutations),
                },
            };
        };
    });
};
exports.createRequireRenameProvider = createRequireRenameProvider;
//# sourceMappingURL=createRequireRenameProvider.js.map