"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.collectExistingTypingPackages = void 0;
const mz_1 = require("mz");
const os_1 = require("os");
const errors_1 = require("../../../shared/errors");
const collectExistingTypingPackages = async (options, packagePath) => {
    const allDependencies = await tryCollectAllDependencies(packagePath);
    if (typeof allDependencies === "string") {
        options.output.stderr(`${os_1.EOL}Error trying to collect existing dependencies for --packageMissingTypes:`);
        options.output.stderr(`\t${allDependencies}`);
        options.output.stderr(os_1.EOL);
        return undefined;
    }
    return Array.from(allDependencies)
        .filter((dependency) => dependency.startsWith("@types/"))
        .map((dependency) => dependency.substring("@types/".length));
};
exports.collectExistingTypingPackages = collectExistingTypingPackages;
const tryCollectAllDependencies = async (packagePath) => {
    try {
        return collectAllDependencies(packagePath);
    }
    catch (error) {
        return errors_1.getQuickErrorSummary(error);
    }
};
const collectAllDependencies = async (packagePath) => {
    const rawContents = (await mz_1.fs.readFile(packagePath)).toString();
    const parsedContents = JSON.parse(rawContents);
    const allDependencies = [];
    for (const groupName of ["dependencies", "devDependencies", "peerDependencies"]) {
        const packageObject = parsedContents[groupName];
        if (packageObject !== undefined) {
            allDependencies.push(...Object.keys(packageObject));
        }
    }
    return new Set(allDependencies);
};
//# sourceMappingURL=collectExistingTypingPackages.js.map