"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.runCommand = void 0;
const chalk_1 = __importDefault(require("chalk"));
const child_process_1 = require("child_process");
const isWindows = () => process.platform === "win32";
const commandAliases = new Map([
    ["npm", isWindows() ? "npm.cmd" : undefined],
    ["yarn", isWindows() ? "yarn.cmd" : undefined],
]);
/**
 * Runs a shell command.
 *
 * @param fullCommand   Command to spawnute, including args.
 * @returns Promise for the result code of the command.
 */
const runCommand = async (options, fullCommand) => {
    const [command, ...args] = typeof fullCommand === "string" ? fullCommand.split(" ") : fullCommand;
    const commandAlias = getCommandAlias(command);
    options.output.stdout(chalk_1.default.grey(`> ${commandAlias} ${args.join(" ")}`));
    return new Promise((resolve, reject) => {
        const childProcess = child_process_1.spawn(commandAlias, args, {
            cwd: options.package.directory,
            stdio: "inherit",
        });
        childProcess.on("error", reject);
        childProcess.on("close", resolve);
    });
};
exports.runCommand = runCommand;
const getCommandAlias = (command) => {
    const alias = commandAliases.get(command);
    return alias === undefined ? command : alias;
};
//# sourceMappingURL=runCommand.js.map